@@ -1,23 +1,24 @@
 {-# OPTIONS_GHC -fglasgow-exts #-}
+{-# LANGUAGE CPP, DeriveDataTypeable #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Network.CGI.Monad
 -- Copyright   :  (c) Bjorn Bringert 2006
 -- License     :  BSD-style
 --
--- Maintainer  :  Anders Kaseorg <andersk@mit.edu>
+-- Maintainer  :  John Chee <cheecheeo@gmail.com>
 -- Stability   :  experimental
 -- Portability :  non-portable
 --
 -- Internal stuff that most people shouldn't have to use.
--- This module mostly deals with the 
+-- This module mostly deals with the
 -- internals of the CGIT monad transformer.
 --
 -----------------------------------------------------------------------------
 
 module Network.CGI.Monad (
   -- * CGI monad class
-  MonadCGI(..), 
+  MonadCGI(..),
   -- * CGI monad transformer
   CGIT(..), CGI,
   runCGIT,
@@ -28,20 +29,20 @@ module Network.CGI.Monad (
  ) where
 
 import Prelude hiding (catch)
-#if MIN_VERSION_base(4,0,0)
-import Control.Exception
-#else
-import Control.Exception.Extensible
-#endif
-  as Exception (SomeException, throwIO)
+import Control.Exception as Exception (SomeException)
+import Control.Applicative (Applicative(..))
 import Control.Monad (liftM)
-import Control.Monad.CatchIO (MonadCatchIO, block, catch, try, unblock)
-import Control.Monad.Error (MonadError(..))
+import Control.Monad.Catch (MonadCatch, MonadThrow, MonadMask, throwM, catch, try, mask, uninterruptibleMask)
+import Control.Monad.Except (MonadError(..))
 import Control.Monad.Reader (ReaderT(..), asks)
 import Control.Monad.Writer (WriterT(..), tell)
 import Control.Monad.Trans (MonadTrans, MonadIO, liftIO, lift)
-import Data.Typeable (Typeable(..), Typeable1(..), 
+#if MIN_VERSION_base(4,7,0)
+import Data.Typeable
+#else
+import Data.Typeable (Typeable(..), Typeable1(..),
                       mkTyConApp, mkTyCon)
+#endif
 
 import Network.CGI.Protocol
 
@@ -55,14 +56,22 @@ type CGI a = CGIT IO a
 
 -- | The CGIT monad transformer.
 newtype CGIT m a = CGIT { unCGIT :: ReaderT CGIRequest (WriterT Headers m) a }
+#if MIN_VERSION_base(4,7,0)
+			deriving (Typeable)
 
+#else
 instance (Typeable1 m, Typeable a) => Typeable (CGIT m a) where
-    typeOf _ = mkTyConApp (mkTyCon "Network.CGI.Monad.CGIT") 
+    typeOf _ = mkTyConApp (mkTyCon "Network.CGI.Monad.CGIT")
                 [typeOf1 (undefined :: m a), typeOf (undefined :: a)]
+#endif
 
 instance (Functor m, Monad m) => Functor (CGIT m) where
     fmap f c = CGIT (fmap f (unCGIT c))
 
+instance (Applicative m, Monad m) => Applicative (CGIT m) where
+    pure = CGIT . pure
+    f <*> x = CGIT (unCGIT f <*> unCGIT x)
+
 instance Monad m => Monad (CGIT m) where
     c >>= f = CGIT (unCGIT c >>= unCGIT . f)
     return = CGIT . return
@@ -72,10 +81,15 @@ instance Monad m => Monad (CGIT m) where
 instance MonadIO m => MonadIO (CGIT m) where
     liftIO = lift . liftIO
 
-instance MonadCatchIO m => MonadCatchIO (CGIT m) where
+instance MonadThrow m => MonadThrow (CGIT m) where
+    throwM e = CGIT . throwM $ e
+
+instance MonadCatch m => MonadCatch (CGIT m) where
     CGIT m `catch` h = CGIT $ m `catch` (unCGIT . h)
-    block (CGIT m) = CGIT (block m)
-    unblock (CGIT m) = CGIT (unblock m)
+
+instance MonadMask m => MonadMask (CGIT m) where
+    mask a = CGIT $ mask $ \u -> unCGIT $ a $ CGIT . u . unCGIT
+    uninterruptibleMask a = CGIT $ uninterruptibleMask $ \u -> unCGIT $ a $ CGIT . u . unCGIT
 
 -- | The class of CGI monads. Most CGI actions can be run in
 --   any monad which is an instance of this class, which means that
@@ -103,26 +117,26 @@ runCGIT (CGIT c) = liftM (uncurry (flip (,))) . runWriterT . runReaderT c
 -- * Error handling
 --
 
-instance MonadCatchIO m => MonadError SomeException (CGIT m) where
-    throwError = throwCGI
-    catchError = catchCGI
+instance MonadCatch m => MonadError SomeException (CGIT m) where
+    throwError = throwM
+    catchError = catch
 
 -- | Throw an exception in a CGI monad. The monad is required to be
---   a 'MonadIO', so that we can use 'throwIO' to guarantee ordering.
-throwCGI :: (MonadCGI m, MonadIO m) => SomeException -> m a
-throwCGI = liftIO . throwIO
+--   a 'MonadThrow', so that we can use 'throwM' to guarantee ordering.
+throwCGI :: (MonadCGI m, MonadThrow m) => SomeException -> m a
+throwCGI = throwM
 
--- | Catches any expection thrown by a CGI action, and uses the given 
+-- | Catches any expection thrown by a CGI action, and uses the given
 --   exception handler if an exception is thrown.
-catchCGI :: (MonadCGI m, MonadCatchIO m) => m a -> (SomeException -> m a) -> m a
+catchCGI :: (MonadCGI m, MonadCatch m) => m a -> (SomeException -> m a) -> m a
 catchCGI = catch
 
 -- | Catches any exception thrown by an CGI action, and returns either
 --   the exception, or if no exception was raised, the result of the action.
-tryCGI :: (Functor m, MonadCGI m, MonadCatchIO m) => m a -> m (Either SomeException a)
+tryCGI :: (Functor m, MonadCGI m, MonadCatch m) => m a -> m (Either SomeException a)
 tryCGI = try
 
 {-# DEPRECATED handleExceptionCGI "Use catchCGI instead." #-}
 -- | Deprecated version of 'catchCGI'. Use 'catchCGI' instead.
-handleExceptionCGI :: (MonadCGI m, MonadCatchIO m) => m a -> (SomeException -> m a) -> m a
+handleExceptionCGI :: (MonadCGI m, MonadCatch m) => m a -> (SomeException -> m a) -> m a
 handleExceptionCGI = catchCGI
